/*-
 * audiosock - skeleton interface to vat audio socket (by Steve McCanne)
 * $Header: audiosock.c,v 1.2 94/05/01 19:10:31 van Exp $ (LBL)
 *
 * The vat audio conferencing program can do its audio input/output
 * from a unix domain socket rather than from the local machine's
 * audio hardware (the `-U sockname' flag tells vat to use `sockname'
 * for its audio).  This is a skeleton program to demonstrate how one
 * might interface to the vat audio socket.  All it does is copy
 * audio data from /dev/audio to the socket and socket data to /dev/audio
 * (so it is a particularly inefficient way to do what vat does anyway
 * if you don't say '-U') but it should be obvious how to modify this
 * program to replace the /dev/audio interface with something appropriate
 * to your audio interface needs.
 *
 * To run this program do something like:
 *	audiosock -U /tmp/.$$.audio &
 *	sleep 1
 *	vat -U /tmp/.$$.audio ...
 * (the 'sleep 1' is to make sure the socket gets created before
 * starting vat).
 *
 *
 * Copyright (c) 1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#include <stropts.h>

int
readblk(int fd, char *cp, int len)
{
	while (len > 0) {
		int cc = read(fd, cp, len);
		if (cc < 0) {
			perror("read");
			exit(1);
		}
		if (cc == 0)
			return (-1);
		cp += cc;
		len -= cc;
	}
	return (0);
}

void
paste(int vat, int audio)
{
	fd_set all_fds;
	int width;

	FD_ZERO(&all_fds);
	FD_SET(vat, &all_fds);
	FD_SET(audio, &all_fds);
	width = ((vat > audio) ? vat : audio) + 1;

	/* flush input to get rid of any data fragments */
	ioctl(audio, I_FLUSH, FLUSHR);

	for (;;) {
		char buffer[160];

		fd_set rfds = all_fds;
		int n = select(width, &rfds, 0, 0, 0);
		if (n < 0) {
			perror("select");
			continue;
		}
		if (FD_ISSET(audio, &rfds)) {
			readblk(audio, buffer, sizeof(buffer));
			(void)write(vat, buffer, sizeof(buffer));
		}
		if (FD_ISSET(vat, &rfds)) {
			if (readblk(vat, buffer, sizeof(buffer)) < 0) {
				close(vat);
				close(audio);
				return;
			}
			(void)write(audio, buffer, sizeof(buffer));
		}
	}
}

char *filename = "/tmp/vatsock";

adios()
{
	(void)unlink(filename);
	exit(0);
}

main(int argc, char **argv)
{
	int op, s;
	struct sockaddr_un sa;
	
	extern char *optarg;
	extern int optind, opterr;

	opterr = 0;
	while ((op = getopt(argc, argv, "U:")) != -1) {
		switch (op) {
		case 'U':
			filename = optarg;
			break;
		}
	}

	(void)signal(SIGTERM, adios);
	(void)signal(SIGINT, adios);
	(void)signal(SIGHUP, adios);

	sigblock(sigmask(SIGPIPE));

	s = socket(AF_UNIX, SOCK_STREAM, 0);
	if (s < 0) {
		perror("socket");
		exit(1);
	}
	sa.sun_family = AF_UNIX;
	strcpy(sa.sun_path, filename);
	if (bind(s, &sa, strlen(sa.sun_path) + sizeof(sa.sun_family)) < 0) {
		perror("bind");
		exit(1);
	}
	if (listen(s, 1) < 0) {
		perror("listen");
		exit(1);
	}

	for (;;) {
		int vat, audio;
		struct sockaddr_un dst;
		int dlen = sizeof(dst);

		vat = accept(s, &dst, &dlen);
		if (vat < 0) {
			perror("accept");
			exit(1);
		}
		audio = open("/dev/audio", O_RDWR);
		if (audio < 0) {
			perror("/dev/audio");
			exit(1);
		}
		paste(vat, audio);
	}
	exit(0);
}
